use axum::extract::{Query, State};
use cairo::{freetype::Face, Context, FontFace, Format, ImageSurface};
use serde::Deserialize;

use crate::{color::Color, polygon, text::DrawableText, SharedState};

#[derive(Deserialize, Default)]
#[allow(non_camel_case_types)]
enum LogoVariant {
    NoText,
    #[default]
    DarkText,
    LightText,
}

#[derive(Deserialize, Default)]
enum LogoOrientation {
    #[default]
    Landscape,
    Portrait,
}

#[derive(Deserialize, Default)]
pub struct ImageProperties {
    #[serde(default)]
    variant: LogoVariant,
    #[serde(default)]
    orientation: LogoOrientation,
}

fn get_surface_size(properties: &ImageProperties) -> (i32, i32) {
    match (&properties.variant, &properties.orientation) {
        (LogoVariant::NoText, _) => (400, 400),
        (_, LogoOrientation::Landscape) => (2127, 591),
        (_, LogoOrientation::Portrait) => (1654, 1654),
    }
}

fn create_image_surface(properties: &ImageProperties) -> ImageSurface {
    let (width, height) = get_surface_size(properties);
    ImageSurface::create(Format::ARgb32, width, height).unwrap()
}

fn draw_logo(context: &Context, properties: &ImageProperties) {
    let (logo_coordinates, logo_outer_radius, logo_inner_radius) =
        match (&properties.variant, &properties.orientation) {
            (LogoVariant::NoText, _) => ((200.0, 200.0), 148.0, 67.0),
            (_, LogoOrientation::Landscape) => ((412.0, 299.0), 209.0, 99.0),
            (_, LogoOrientation::Portrait) => ((828.0, 563.0), 253.0, 118.0),
        };

    polygon::draw_polygon_of_segmented_polygons(
        logo_coordinates,
        logo_outer_radius,
        logo_inner_radius,
        6,
        &context,
    )
    .unwrap();
}

fn draw_text(
    context: &Context,
    font_regular: &Face,
    font_light: &Face,
    properties: &ImageProperties,
) {
    if let LogoVariant::NoText = properties.variant {
        return;
    }

    let color = match &properties.variant {
        LogoVariant::DarkText => Color::from("#383F50"),
        LogoVariant::LightText => Color::from("#ffffff"),
        _ => unreachable!(),
    };

    context.set_source_rgb(
        color.r as f64 / 255.0,
        color.g as f64 / 255.0,
        color.b as f64 / 255.0,
    );

    let texts = match &properties.orientation {
        LogoOrientation::Landscape => (
            DrawableText {
                position: (760.0, 356.0),
                size: 228.0,
                spacing: 45.5,
                text: "makerlab",
            },
            DrawableText {
                position: (1495.0, 504.0),
                size: 110.0,
                spacing: 10.5,
                text: "MURNAU",
            },
        ),
        LogoOrientation::Portrait => (
            DrawableText {
                position: (48.0, 1242.0),
                size: 280.0,
                spacing: 53.5,
                text: "makerlab",
            },
            DrawableText {
                position: (493.0, 1424.0),
                size: 135.0,
                spacing: 13.0,
                text: "MURNAU",
            },
        ),
    };

    context.set_font_face(&FontFace::create_from_ft(font_regular).unwrap());
    texts.0.draw(&context);

    context.set_font_face(&FontFace::create_from_ft(font_light).unwrap());
    texts.1.draw(&context);
}

#[axum_macros::debug_handler]
pub async fn png(
    Query(properties): Query<ImageProperties>,
    State(state): State<SharedState>,
) -> impl axum::response::IntoResponse {
    let (font_regular, font_light) = state.fonts.get().await;

    // cannot use await after this, because surface does not implement Send
    let surface = create_image_surface(&properties);
    let context = Context::new(&surface).unwrap();

    draw_logo(&context, &properties);
    draw_text(&context, &font_regular, &font_light, &properties);

    let mut data: Vec<u8> = Vec::new();
    surface.write_to_png(&mut data).unwrap();

    (
        axum::response::AppendHeaders([(axum::http::header::CONTENT_TYPE, "image/png")]),
        data,
    )
}
